@@ -230,10 +230,10 @@ protected:
 
     memory_desc_t ws_md_;
 
-    rnn_pd_t(const rnn_desc_t *adesc, const primitive_attr_t *attr,
+    rnn_pd_t(const op_desc_t *adesc, const primitive_attr_t *attr,
             const rnn_fwd_pd_t *hint_fwd_pd)
         : primitive_desc_t(attr, base_pkind)
-        , desc_(*adesc)
+        , desc_(*op_desc_t::to_desc<rnn_desc_t>(adesc))
         , hint_fwd_pd_(hint_fwd_pd)
         , src_layer_md_(desc_.src_layer_desc)
         , src_iter_md_(desc_.src_iter_desc)
@@ -323,7 +323,7 @@ struct rnn_fwd_pd_t : public rnn_pd_t {
     }
 
 protected:
-    rnn_fwd_pd_t(const rnn_desc_t *adesc, const primitive_attr_t *attr,
+    rnn_fwd_pd_t(const op_desc_t *adesc, const primitive_attr_t *attr,
             const rnn_fwd_pd_t *hint_fwd_pd)
         : rnn_pd_t(adesc, attr, hint_fwd_pd) {}
 };
@@ -521,7 +521,7 @@ protected:
     memory_desc_t diff_dst_iter_md_;
     memory_desc_t diff_dst_iter_c_md_;
 
-    rnn_bwd_pd_t(const rnn_desc_t *adesc, const primitive_attr_t *attr,
+    rnn_bwd_pd_t(const op_desc_t *adesc, const primitive_attr_t *attr,
             const rnn_fwd_pd_t *hint_fwd_pd)
         : rnn_pd_t(adesc, attr, hint_fwd_pd)
         , diff_src_layer_md_(desc_.diff_src_layer_desc)
