@@ -20,7 +20,7 @@ limitations under the License.
 #define EIGEN_USE_GPU
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 
-#include "third_party/eigen3/unsupported/Eigen/CXX11/Tensor"
+#include "unsupported/Eigen/CXX11/Tensor"  // from @eigen_archive
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/register_types.h"
 #include "tensorflow/core/framework/tensor.h"
@@ -39,35 +39,37 @@ limitations under the License.
 #include "tensorflow/core/util/tensor_ops_util.h"
 #include "tensorflow/core/util/util.h"
 
-// stream.h isn't available in some platforms such as Android, iOS, and
-// ChromiumOS. Only include it for platforms that PluggableDevice is tested on.
+// stream.h isn't available in some platforms such as Android, iOS, ChromiumOS,
+// and Fuchsia. Only include it for platforms that PluggableDevice is tested on.
 #if !defined(PLUGGABLE_DEVICE_SUPPORTED) &&                              \
     (__x86_64__ || __i386__ || defined(__APPLE__) || defined(_WIN32)) && \
     !defined(ANDROID) && !defined(__ANDROID__) && !TARGET_OS_IOS &&      \
-    !defined(PLATFORM_CHROMIUMOS)
+    !defined(PLATFORM_CHROMIUMOS) && !defined(__Fuchsia__)
 #define PLUGGABLE_DEVICE_SUPPORTED
 #endif
 
 #ifdef PLUGGABLE_DEVICE_SUPPORTED
-#include "tensorflow/stream_executor/stream.h"
+#include "xla/stream_executor/stream.h"
 #endif
 
 namespace tensorflow {
 
 typedef Eigen::ThreadPoolDevice CPUDevice;
 
-Status TensorShapeFromTensor(const Tensor& t, PartialTensorShape* out);
+absl::Status TensorShapeFromTensor(const Tensor& t, PartialTensorShape* out);
 
-Status GetElementShapeFromInput(OpKernelContext* c,
-                                const TensorList& tensor_list, int index,
-                                PartialTensorShape* element_shape);
+absl::Status GetElementShapeFromInput(OpKernelContext* c,
+                                      const TensorList& tensor_list, int index,
+                                      PartialTensorShape* element_shape);
 
-Status GetInputList(OpKernelContext* c, int index, const TensorList** list);
+absl::Status GetInputList(OpKernelContext* c, int index,
+                          const TensorList** list);
 
-Status ForwardInputOrCreateNewList(OpKernelContext* c, int32_t input_index,
-                                   int32_t output_index,
-                                   const TensorList& input_list,
-                                   TensorList** output_list);
+absl::Status ForwardInputOrCreateNewList(OpKernelContext* c,
+                                         int32_t input_index,
+                                         int32_t output_index,
+                                         const TensorList& input_list,
+                                         TensorList** output_list);
 
 // TODO(penporn): Move this to a proper place.
 inline bool IsPluggableDevice(OpKernelContext* c) {
@@ -81,7 +83,7 @@ inline void SetZero(OpKernelContext* ctx, Tensor& tensor) {
     auto ptr =
         se::DeviceMemoryBase(tensor.flat<T>().data(), tensor.TotalBytes());
     auto stream = ctx->op_device_context()->stream();
-    auto result = stream->ThenMemZero(&ptr, tensor.TotalBytes()).ok();
+    auto result = stream->MemZero(&ptr, tensor.TotalBytes()).ok();
     DCHECK_EQ(true, result);
   } else {
 #endif  // PLUGGABLE_DEVICE_SUPPORTED
@@ -102,7 +104,7 @@ inline void CopyTensorPluggableDevice(OpKernelContext* ctx, Tensor& src,
   auto src_ptr = se::DeviceMemoryBase(src_t.data(), src.TotalBytes());
   auto dst_ptr = se::DeviceMemoryBase(dst_t.data(), dst.TotalBytes());
   auto stream = ctx->op_device_context()->stream();
-  auto result = stream->ThenMemcpy(&dst_ptr, src_ptr, src.TotalBytes()).ok();
+  auto result = stream->Memcpy(&dst_ptr, src_ptr, src.TotalBytes()).ok();
   DCHECK_EQ(true, result);
 #else
   LOG(FATAL)  // Crash OK.
@@ -149,7 +151,8 @@ void ConcatPluggableDevice(
       auto size = sizes[j];
       se::DeviceMemoryBase out_base{out, size * sizeof(T)};
       se::DeviceMemoryBase inp_base{const_cast<T*>(inp[j]), size * sizeof(T)};
-      stream->ThenMemcpy(&out_base, inp_base, size * sizeof(T));
+      OP_REQUIRES_OK(context,
+                     stream->Memcpy(&out_base, inp_base, size * sizeof(T)));
       out += size;
       inp[j] += size;
     }
@@ -395,8 +398,11 @@ class TensorListConcat : public OpKernel {
   void Compute(OpKernelContext* c) override {
     PartialTensorShape element_shape_except_first_dim;
     if (!element_shape_.unknown_rank()) {
-      element_shape_except_first_dim = PartialTensorShape(
-          gtl::ArraySlice<int64_t>(element_shape_.dim_sizes()).subspan(1));
+      auto dim_sizes = element_shape_.dim_sizes();
+      OP_REQUIRES(c, !dim_sizes.empty(),
+                  errors::InvalidArgument("element_shape must not be empty"));
+      element_shape_except_first_dim =
+          PartialTensorShape(absl::Span<const int64_t>(dim_sizes).subspan(1));
     }
     // Check that the input Variant tensor is indeed a TensorList and has the
     // correct element type.
@@ -455,7 +461,7 @@ class TensorListConcat : public OpKernel {
               errors::InvalidArgument("Concat saw a scalar shape at index ", i,
                                       " but requires at least vectors."));
           TensorShape shape_except_first_dim = TensorShape(
-              gtl::ArraySlice<int64_t>(t.shape().dim_sizes()).subspan(1));
+              absl::Span<const int64_t>(t.shape().dim_sizes()).subspan(1));
           OP_REQUIRES_OK(c, tmp.MergeWith(shape_except_first_dim,
                                           &element_shape_except_first_dim));
           OP_REQUIRES(c, first_dim == -1 || first_dim == t.shape().dim_size(0),
@@ -688,6 +694,12 @@ class TensorListGather : public OpKernel {
     if (!tensor_list->element_shape.IsFullyDefined()) {
       for (int index = 0; index < indices.NumElements(); ++index) {
         const int i = indices.flat<int32>()(index);
+
+        OP_REQUIRES(c, 0 <= i && i < tensor_list->tensors().size(),
+                    absl::InvalidArgumentError(absl::StrCat(
+                        "Trying to gather element ", i, " in a list with ",
+                        tensor_list->tensors().size(), " elements.")));
+
         const Tensor& t = tensor_list->tensors()[i];
         if (t.dtype() != DT_INVALID) {
           PartialTensorShape tmp = partial_element_shape;
@@ -815,8 +827,8 @@ class TensorListFromTensor : public OpKernel {
 
 // Scatters values in `value` into `list`. Assumes that `indices` are valid.
 template <typename Device, typename T>
-Status Scatter(OpKernelContext* c, const Tensor& value, const Tensor& indices,
-               TensorList* list) {
+absl::Status Scatter(OpKernelContext* c, const Tensor& value,
+                     const Tensor& indices, TensorList* list) {
   const auto copy_tensor = IsPluggableDevice(c) ? &CopyTensorPluggableDevice<T>
                                                 : &CopyTensor<Device, T>;
   for (int index = 0; index < indices.NumElements(); ++index) {
@@ -836,7 +848,7 @@ Status Scatter(OpKernelContext* c, const Tensor& value, const Tensor& indices,
     copy_tensor(c, tmp, aligned);
     std::swap(list->tensors()[i], aligned);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 template <typename Device, typename T>
@@ -909,7 +921,12 @@ class TensorListScatter : public OpKernel {
     OP_REQUIRES_OK(c, TensorShapeFromTensor(c->input(2), &element_shape));
     // TensorListScatterV2 passes the num_elements input, TensorListScatter does
     // not.
-    int num_elements = c->num_inputs() >= 4 ? c->input(3).scalar<int>()() : -1;
+    int num_elements = -1;
+    if (c->num_inputs() >= 4) {
+      OP_REQUIRES(c, TensorShapeUtils::IsScalar(c->input(3).shape()),
+                  errors::InvalidArgument("num_elements must be a scalar"));
+      num_elements = c->input(3).scalar<int>()();
+    }
     OP_REQUIRES(c, num_elements >= -1,
                 errors::InvalidArgument(
                     "TensorListScatter expects num_elements >= -1, found: ",
@@ -963,14 +980,14 @@ class TensorListScatter : public OpKernel {
 };
 
 template <typename Device>
-Status TensorListBinaryAdd(OpKernelContext* c, const TensorList& a,
-                           const TensorList& b, TensorList* out) {
+absl::Status TensorListBinaryAdd(OpKernelContext* c, const TensorList& a,
+                                 const TensorList& b, TensorList* out) {
   return TensorListBinaryAdd(c, a, b, out, BinaryAddTensors<Device>);
 }
 
 template <typename Device>
-Status TensorListZerosLike(OpKernelContext* c, const TensorList& x,
-                           TensorList* y) {
+absl::Status TensorListZerosLike(OpKernelContext* c, const TensorList& x,
+                                 TensorList* y) {
   return TensorListZerosLike(c, x, y, ZerosLikeTensor<Device>);
 }
 
